# Python imports

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import GLib

# Application imports
from libs.dto.lsp_messages import get_message_str
from libs.dto.lsp_message_structs import MessageTypes



class LogList(Gtk.ListBox):
    def __init__(self):
        super(LogList, self).__init__()

        self.display_logging = False

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        ctx = self.get_style_context()
        ctx.add_class("log-list-listbox")

        self.set_placeholder( Gtk.Label(label="Empty Log List...") )
        self.set_selection_mode( Gtk.SelectionMode.NONE )

    def _setup_signals(self):
        def tggl_row_view(lb, row):
            frame    = row.get_child()
            revealer = frame.get_children()[0]
            self._active_label = revealer.get_children()[0]
            revealer.set_reveal_child( not revealer.get_reveal_child() )

            del frame
            del revealer

        def scroll_down(self, widget):
            GLib.idle_add(self.scroll_down)

        self.connect("row-activated", tggl_row_view)
        self.connect("size-allocate", scroll_down)
        self.connect("button-release-event", self._label_button_release_event)

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        ...

    def toggle_logging(self, state: bool):
        self.display_logging = state

    def add_log_entry(self, m_type: str, data: MessageTypes):
        if not self.display_logging: return

        message   = get_message_str(data)

        frame     = Gtk.Frame()
        revealer  = Gtk.Revealer()
        label     = Gtk.Label(label = message)
        lbl_str   = f"> {m_type} Message: {data.id}" if hasattr(data, "id") else f"> {m_type} Notification"

        label.set_xalign(0.010)
        frame.set_label(lbl_str)
        frame.set_label_align(0.010, 0.5)
        revealer.set_reveal_child(False)

        revealer.add(label)
        frame.add(revealer)
        self.add(frame)

        frame.show_all()

        del message
        del lbl_str

    def clear(self):
        for child in self.get_children():
            self.remove(child)

    def scroll_down(self):
        vadjustment = self.get_parent().get_vadjustment()
        vadjustment.set_value( vadjustment.get_upper())

    def _label_button_release_event(self, log_list, eve):
        if eve.button == 3:
            event_system.emit("copy-to-clipboard", self._active_label.get_label())
            return True
